/*
 * MsgDialog.java - Generalized GUI to handle questions, errors, and messages
 *
 *      Copyright (C) - 2006 Michael Latham
 *      This software may be freely distributed and modified under the GNU 
 *      General Public License, Version 2 or later, as published by the 
 *      Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK.  
 *      The author and/or authors cannot be held responsible for any 
 *      misuse or damage.
 *
 *      NOTE: Please add revisions as file is changed!
 *
 *      REV         DATE                        DESCRIPTION
 *      -----       ----------          --------------------------------------------------
 *      00.00       2006-07-27          Initial Release
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
/**
 * MsgDialog is a generic dialog pop-up to either show a message, error, or get user input
 *
 * USAGE
 * This dialog is modal, and will hide on close.  The caller is responsible for disposal after
 * retrieving any input.  The basic steps to invoke are:
 *  1. Construct
 *  2. Set type (MESSAGE is the default)
 *  3. Add message or question (if applicable)
 *  4. Set visible
 *  5. Get answer (if applicable)
 *  6. Get data
 *  7. Dispose
 *
 * Example of basic message:
 *
 *      MsgDialog msg = new MsgDialog(this);
 *      msg.setType( MsgDialog.MESSAGE );
 *      msg.setData( "This is my message." );
 *      msg.setVisible(true);
 *      msg.dispose();
 *
 * Example of question:
 *
 *      MsgDialog msg = new MsgDialog(this);
 *      msg.setType( MsgDialog.QUESTION );
 *      msg.setData( "Do you want to continue?" );
 *      msg.setVisible(true);
 *      int answer = msg.getAnswer();
 *      msg.dispose();
 *      if (answer == MsgDialog.OK) {
 *          //Do stuff
 *      }
 *
 * Example of user input:
 *
 *      MsgDialog msg = new MsgDialog(this);
 *      msg.setType( MsgDialog.INPUT );
 *      msg.setData( "" );
 *      msg.setVisible(true);
 *      int answer = msg.getAnswer();
 *      // do not dispose, yet
 *      if (answer == MsgDialog.OK) {
 *          String value = msg.getInput();
 *          //Do stuff with value
 *      }
 *      msg.dispose();
 *
 * @author Michael Latham
 */
public class MsgDialog extends JDialog
{
    /** Constant - MsgDialog type */
    public static final int MESSAGE     = 0;
    /** Constant - MsgDialog type */
    public static final int QUESTION    = 1;
    /** Constant - MsgDialog type */
    public static final int INPUT       = 3;
    /** Constant - MsgDialog answer */
    public static final int CANCEL      = 0;
    /** Constant - MsgDialog answer */
    public static final int OK          = 1;

    /** Owner frame of this dialog */
    private Frame m_parent;
    /** Owner dialog of this dialog */
    private Dialog m_parentDlg;
    /** Text area component for input or message */
	private JTextArea m_msgTA;
    /** Screen component */
	private JScrollPane jScrollPane1;
    /** Button to acknowledge */
	private JButton m_okBtn;
    /** Button to cancel */
	private JButton m_cancelBtn;
    /** Screen component */
	private JPanel contentPane;
    /** Data user input if type is INPUT */
    private String  m_input;
    /** Answer from user - Ok or Cancel */
    private int     m_answer;
    /** Mode of MsgDialog - INPUT, QUESTION, or MESSAGE */
    private int     m_modeType;


    /**
     * Constructor where parent is a Frame
     * @param w Parent frame
     */
    public MsgDialog(Frame w)
	{
		super(w);
        m_parent = w;
        m_parentDlg = null;
		initializeComponent();
        postInitialize();
	}

    /**
     * Constructor where parent is a Dialog
     * @param w Parent dialog
     */
	public MsgDialog(Dialog w)
	{
		super(w);
        m_parentDlg = w;
        m_parent = null;
		initializeComponent();
        postInitialize();
	}

	/**
     * Initialize all the components for the GUI
	 */
	private void initializeComponent()
	{
		m_msgTA = new JTextArea();
        m_msgTA.setLineWrap(true);
        m_msgTA.setWrapStyleWord(true);
        m_msgTA.setDisabledTextColor(Color.BLACK);
		jScrollPane1 = new JScrollPane();
		m_okBtn = new JButton();
		m_cancelBtn = new JButton();
		contentPane = (JPanel)this.getContentPane();

		//
		// m_msgTA
		//
		//
		// jScrollPane1
		//
		jScrollPane1.setViewportView(m_msgTA);
		//
		// m_okBtn
		//
		m_okBtn.setText("Ok");
		m_okBtn.setToolTipText("Accept and close dialog");
		m_okBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				m_okBtn_actionPerformed(e);
			}

		});
		//
		// m_cancelBtn
		//
		m_cancelBtn.setText("Cancel");
		m_cancelBtn.setToolTipText("Close dialog");
		m_cancelBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				m_cancelBtn_actionPerformed(e);
			}

		});
		//
		// contentPane
		//
		contentPane.setLayout(null);
		addComponent(contentPane, jScrollPane1, 5,6,370,90);
		addComponent(contentPane, m_okBtn, 85,105,83,28);
		addComponent(contentPane, m_cancelBtn, 218,105,83,28);
		//
		// MsgDialog
		//
        Point p;
        if (m_parent == null)
            p = QETools.getNewLocation( m_parentDlg );
        else
            p = QETools.getNewLocation( m_parent );
		this.setLocation(p);
		this.setTitle("Info Exchange");
		this.setSize(new Dimension(389, 171));
		this.setModal(true);
		this.setResizable(false);
	}
    
    /**
     * Post initialization default setup
     */
    private void postInitialize() {
        setType( MESSAGE );
    }

	/** 
     * Add Component Without a Layout Manager (Absolute Positioning)
     * @param container Container to which to add component
     * @param c         Component to add to container
     * @param x         X coordinate position for component in container
     * @param y         Y coordinate position for component in container
     * @param width     width of component
     * @param height    height of component
     */
	private void addComponent(Container container,Component c,int x,int y,int width,int height)
	{
		c.setBounds(x,y,width,height);
		container.add(c);
	}

	/**
     * OK Button action handler
     * @param e Action event object
     */
    private void m_okBtn_actionPerformed(ActionEvent e)
	{
        m_answer = this.OK;
        m_input  = m_msgTA.getText();
        setVisible(false);
	}

	/**
     * Cancel Button action handler
     * @param e Action event object
     */
	private void m_cancelBtn_actionPerformed(ActionEvent e)
	{
        m_answer = this.CANCEL;
        m_input  = m_msgTA.getText();
        setVisible(false);
	}
    
    /**
     * Retrieve the answer; use after dialog is released by Ok or Cancel action
     * @return answer = this.OK or this.CANCEL
     */
    public int getAnswer() {
        return m_answer;
    }
    
    /**
     * Retrieve user input; use after dialog is released by Ok or Cancel action
     * @return Data user entered
     */
    public String getInput() {
        return m_input;
    }
    
    /**
     * Set the mode type for this MsgDialog
     * @param modeType  QUESTION, MESSAGE, or INPUT
     */
    public void setType( int modeType ) {
        m_modeType = modeType;
        
        switch( m_modeType ) {
            case INPUT:
                m_msgTA.setEnabled(true);
                break;
            default:
                m_msgTA.setEnabled(false);
                break;
        }
    }
    
    /**
     * Set message or question to be displayed and acknowledged
     * @param dataStr   Message or question
     */
    public void setData( String dataStr ) {
        if (dataStr == null) {
            m_msgTA.setText( "" );
            return;
        }
        m_msgTA.setText( dataStr );
    }

}
